@@ -50,9 +50,6 @@
 package com.openexchange.push.dovecot;
 
 import static com.openexchange.java.Autoboxing.I;
-import java.net.URI;
-import java.net.URISyntaxException;
-import java.util.Collection;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -61,17 +58,14 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutionException;
 import org.slf4j.Logger;
 import com.hazelcast.core.HazelcastInstance;
-import com.hazelcast.core.Hazelcasts;
 import com.hazelcast.core.Member;
-import com.openexchange.config.ConfigurationService;
 import com.openexchange.dovecot.doveadm.client.DoveAdmClient;
 import com.openexchange.exception.OXException;
+import com.openexchange.hazelcast.Hazelcasts;
 import com.openexchange.java.Streams;
-import com.openexchange.java.Strings;
 import com.openexchange.lock.AccessControl;
 import com.openexchange.lock.LockService;
 import com.openexchange.lock.ReentrantLockAccessControl;
-import com.openexchange.push.PushExceptionCodes;
 import com.openexchange.push.PushListener;
 import com.openexchange.push.PushListenerService;
 import com.openexchange.push.PushManagerExtendedService;
@@ -86,8 +80,9 @@ import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.ObfuscatorService;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionMatcher;
 import com.openexchange.sessiond.SessiondService;
-import com.openexchange.sessionstorage.hazelcast.serialization.PortableMultipleSessionRemoteLookUp;
+import com.openexchange.sessionstorage.hazelcast.serialization.PortableMultipleActiveSessionRemoteLookUp;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableSession;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableSessionCollection;
 
@@ -117,8 +112,8 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
      * @return The new instance
      * @throws OXException If initialization fails
      */
-    public static DovecotPushManagerService newInstance(String endPoint, DovecotPushClusterLock clusterLock, ServiceLookup services) throws OXException {
-        DovecotPushManagerService newi = new DovecotPushManagerService(endPoint, clusterLock, services);
+    public static DovecotPushManagerService newInstance(DovecotPushClusterLock clusterLock, ServiceLookup services) {
+        DovecotPushManagerService newi = new DovecotPushManagerService(clusterLock, services);
         instance = newi;
         return newi;
     }
@@ -149,45 +144,18 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
     private final ServiceLookup services;
     private final Map<SimpleKey, DovecotPushListener> listeners;
     private final DovecotPushClusterLock clusterLock;
-    private final String authLogin;
-    private final String authPassword;
-    private final URI uri;
     private final AccessControl globalLock;
 
     /**
      * Initializes a new {@link DovecotPushManagerService}.
      */
-    private DovecotPushManagerService(String endPoint, DovecotPushClusterLock clusterLock, ServiceLookup services) throws OXException {
+    private DovecotPushManagerService(DovecotPushClusterLock clusterLock, ServiceLookup services) {
         super();
         name = "Dovecot Push Manager";
         this.services = services;
         this.clusterLock = clusterLock;
         listeners = new ConcurrentHashMap<SimpleKey, DovecotPushListener>(512, 0.9F, 1);
 
-        // Parse auth data
-        {
-            ConfigurationService service = services.getService(ConfigurationService.class);
-            String authLogin = service.getProperty("com.openexchange.rest.services.basic-auth.login");
-            String authPassword = service.getProperty("com.openexchange.rest.services.basic-auth.password");
-            if (Strings.isEmpty(authLogin) || Strings.isEmpty(authPassword)) {
-                LOGGER.error("Denied initialization due to unset Basic-Auth configuration. Please set properties 'com.openexchange.rest.services.basic-auth.login' and 'com.openexchange.rest.services.basic-auth.password' appropriately.");
-                throw ServiceExceptionCode.absentService(ConfigurationService.class);
-            }
-            this.authLogin = authLogin.trim();
-            this.authPassword = authPassword.trim();
-        }
-
-        // Parse URL
-        if (Strings.isEmpty(endPoint)) {
-            uri = null;
-        } else {
-            try {
-                uri = new URI(endPoint);
-            } catch (URISyntaxException e) {
-                throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, null == endPoint ? "<empty>" : endPoint);
-            }
-        }
-
         // The fall-back lock
         globalLock = new ReentrantLockAccessControl();
     }
@@ -268,19 +236,37 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
         // Look-up sessions
         SessiondService sessiondService = services.getService(SessiondService.class);
         if (null != sessiondService) {
-            String oldSessionId = null == optOldSession ? null : optOldSession.getSessionID();
+            final String oldSessionId = null == optOldSession ? null : optOldSession.getSessionID();
 
             // Query local ones
-            Collection<Session> sessions = sessiondService.getSessions(userId, contextId);
-            for (Session session : sessions) {
-                if ((null == oldSessionId || false == oldSessionId.equals(session.getSessionID())) && PushUtility.allowedClient(session.getClient(), session, true)) {
-                    return session;
+            SessionMatcher matcher = new SessionMatcher() {
+
+                @Override
+                public Set<Flag> flags() {
+                    return SessionMatcher.ALL_FLAGS;
+                }
+
+                @Override
+                public boolean accepts(Session session) {
+                    return (oldSessionId == null || !oldSessionId.equals(session.getSessionID())) && PushUtility.allowedClient(session.getClient(), session, true);
+                }
+            };
+            Session anotherActiveSession = sessiondService.findFirstMatchingSessionForUser(userId, contextId, matcher);
+            if (anotherActiveSession != null) {
+                return anotherActiveSession;
+            }
+
+            // Look-up remote sessions, too, if possible
+            Session session = lookUpRemoteSessionFor(userId, contextId, optOldSession);
+            if (session != null) {
+                Session ses = sessiondService.getSession(session.getSessionID());
+                if (ses != null) {
+                    return ses;
                 }
             }
         }
 
-        // Look-up remote sessions, too, if possible
-        return lookUpRemoteSessionFor(userId, contextId, optOldSession);
+        return null;
     }
 
     private Session lookUpRemoteSessionFor(int userId, int contextId, Session optOldSession) {
@@ -314,7 +300,7 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
             }
         };
         try {
-            return Hazelcasts.executeByMembersAndFilter(new PortableMultipleSessionRemoteLookUp(userId, contextId), otherMembers, hzInstance.getExecutorService("default"), filter);
+            return Hazelcasts.executeByMembersAndFilter(new PortableMultipleActiveSessionRemoteLookUp(userId, contextId), otherMembers, hzInstance.getExecutorService("default"), filter);
         } catch (ExecutionException e) {
             Throwable cause = e.getCause();
             if (cause instanceof RuntimeException) {
@@ -336,7 +322,7 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
      * @throws OXException If operation fails
      */
     public DovecotPushListener injectAnotherListenerUsing(RegistrationContext registrationContext, boolean permanent) {
-        DovecotPushListener listener = new DovecotPushListener(uri, authLogin, authPassword, registrationContext, permanent, this, services);
+        DovecotPushListener listener = new DovecotPushListener(registrationContext, permanent, this, services);
         // Replace old/existing one
         listeners.put(SimpleKey.valueOf(registrationContext.getUserId(), registrationContext.getContextId()), listener);
         return listener;
@@ -465,22 +451,34 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
                 try {
                     lock.acquireGrant();
                     if (false == listeners.containsKey(key)) {
-                        DovecotPushListener listener = new DovecotPushListener(uri, authLogin, authPassword, registrationContext, false, this, services);
+                        DovecotPushListener listener = new DovecotPushListener(registrationContext, false, this, services);
                         listeners.put(key, listener);
                         removeListener = true;
                         String reason = listener.initateRegistration();
                         if (null == reason) {
                             removeListener = false;
                             unlock = false;
-                            LOGGER.info("Started Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                            if (LOGGER.isDebugEnabled()) {
+                                LOGGER.debug("Started Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient(), new Throwable("Dovecot start listener trace"));
+                            } else {
+                                LOGGER.info("Started Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                            }
                             return listener;
                         }
 
                         // Registration failed
-                        LOGGER.info("Could not register Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                        if (LOGGER.isDebugEnabled()) {
+                            LOGGER.debug("Could not register Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient(), new Throwable("Dovecot start listener trace"));
+                        } else {
+                            LOGGER.info("Could not register Dovecot listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                        }
                     } else {
                         // Already running for session user
-                        LOGGER.info("Did not start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                        if (LOGGER.isDebugEnabled()) {
+                            LOGGER.debug("Did not start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient(), new Throwable("Dovecot start listener trace"));
+                        } else {
+                            LOGGER.info("Did not start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
+                        }
                     }
                 } catch (InterruptedException e) {
                     Thread.currentThread().interrupt();
@@ -497,7 +495,11 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
                 }
             }
         } else {
-            LOGGER.info("Could not acquire lock to start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Could not acquire lock to start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient(), new Throwable("Dovecot start listener trace"));
+            } else {
+                LOGGER.info("Could not acquire lock to start Dovecot listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
+            }
         }
 
         // No listener registered for given session
@@ -513,13 +515,25 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
         StopResult stopResult = stopListener(true, false, session.getUserId(), session.getContextId());
         switch (stopResult) {
         case RECONNECTED:
-            LOGGER.info("Reconnected Dovecot listener for user {} in context {} using another session", I(session.getUserId()), I(session.getContextId()));
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Reconnected Dovecot listener for user {} in context {} using another session", I(session.getUserId()), I(session.getContextId()), new Throwable("Dovecot stop listener trace"));
+            } else {
+                LOGGER.info("Reconnected Dovecot listener for user {} in context {} using another session", I(session.getUserId()), I(session.getContextId()));
+            }
             return true;
         case RECONNECTED_AS_PERMANENT:
-            LOGGER.info("Reconnected as permanent Dovecot listener for user {} in context {}", I(session.getUserId()), I(session.getContextId()));
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Reconnected as permanent Dovecot listener for user {} in context {}", I(session.getUserId()), I(session.getContextId()), new Throwable("Dovecot stop listener trace"));
+            } else {
+                LOGGER.info("Reconnected as permanent Dovecot listener for user {} in context {}", I(session.getUserId()), I(session.getContextId()));
+            }
             return true;
         case STOPPED:
-            LOGGER.info("Stopped Dovecot listener for user {} in context {} with session {}", I(session.getUserId()), I(session.getContextId()), session.getSessionID());
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Stopped Dovecot listener for user {} in context {} with session {}", I(session.getUserId()), I(session.getContextId()), session.getSessionID(), new Throwable("Dovecot stop listener trace"));
+            } else {
+                LOGGER.info("Stopped Dovecot listener for user {} in context {} with session {}", I(session.getUserId()), I(session.getContextId()), session.getSessionID());
+            }
             return true;
         default:
             break;
@@ -578,14 +592,18 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
                     lock.acquireGrant();
                     DovecotPushListener current = listeners.get(key);
                     if (null == current) {
-                        DovecotPushListener listener = new DovecotPushListener(uri, authLogin, authPassword, registrationContext, true, this, services);
+                        DovecotPushListener listener = new DovecotPushListener(registrationContext, true, this, services);
                         listeners.put(key, listener);
                         removeListener = true;
                         String reason = listener.initateRegistration();
                         if (null == reason) {
                             removeListener = false;
                             unlock = false;
-                            LOGGER.info("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                            if (LOGGER.isDebugEnabled()) {
+                                LOGGER.debug("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId), new Throwable("Dovecot start permanent listener trace"));
+                            } else {
+                                LOGGER.info("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                            }
                             return listener;
                         }
 
@@ -594,22 +612,34 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
                     } else if (!current.isPermanent()) {
                         // Cancel current & replace
                         current.unregister(false);
-                        DovecotPushListener listener = new DovecotPushListener(uri, authLogin, authPassword, registrationContext, true, this, services);
+                        DovecotPushListener listener = new DovecotPushListener(registrationContext, true, this, services);
                         listeners.put(key, listener);
                         removeListener = true;
                         String reason = listener.initateRegistration();
                         if (null == reason) {
                             removeListener = false;
                             unlock = false;
-                            LOGGER.info("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                            if (LOGGER.isDebugEnabled()) {
+                                LOGGER.debug("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId), new Throwable("Dovecot start permanent listener trace"));
+                            } else {
+                                LOGGER.info("Started permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                            }
                             return listener;
                         }
 
                         // Registration failed
-                        LOGGER.info("Could not register permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                        if (LOGGER.isDebugEnabled()) {
+                            LOGGER.debug("Could not register permanent Dovecot listener for user {} in context {}", I(userId), I(contextId), new Throwable("Dovecot start permanent listener trace"));
+                        } else {
+                            LOGGER.info("Could not register permanent Dovecot listener for user {} in context {}", I(userId), I(contextId));
+                        }
                     } else {
                         // Already running for session user
-                        LOGGER.info("Did not start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId));
+                        if (LOGGER.isDebugEnabled()) {
+                            LOGGER.debug("Did not start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId), new Throwable("Dovecot start permanent listener trace"));
+                        } else {
+                            LOGGER.info("Did not start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId));
+                        }
                     }
                 } catch (InterruptedException e) {
                     Thread.currentThread().interrupt();
@@ -626,7 +656,11 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
                 }
             }
         } else {
-            LOGGER.info("Could not acquire lock to start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId));
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Could not acquire lock to start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId), new Throwable("Dovecot start permanent listener trace"));
+            } else {
+                LOGGER.info("Could not acquire lock to start permanent Dovecot listener for user {} in context {} as there is already an associated listener", I(userId), I(contextId));
+            }
         }
 
         // No listener registered for given session
@@ -642,10 +676,18 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
         StopResult stopResult = stopListener(tryToReconnect, true, pushUser.getUserId(), pushUser.getContextId());
         switch (stopResult) {
         case RECONNECTED:
-            LOGGER.info("Reconnected permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()));
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Reconnected permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()), new Throwable("Dovecot stop permanent listener trace"));
+            } else {
+                LOGGER.info("Reconnected permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()));
+            }
             return true;
         case STOPPED:
-            LOGGER.info("Stopped permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()));
+            if (LOGGER.isDebugEnabled()) {
+                LOGGER.debug("Stopped permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()), new Throwable("Dovecot stop permanent listener trace"));
+            } else {
+                LOGGER.info("Stopped permanent Dovecot listener for user {} in context {}", I(pushUser.getUserId()), I(pushUser.getContextId()));
+            }
             return true;
         default:
             break;
@@ -655,7 +697,7 @@ public class DovecotPushManagerService implements PushManagerExtendedService {
     }
 
     @Override
-    public List<PushUserInfo> getAvailablePushUsers() throws OXException {
+    public List<PushUserInfo> getAvailablePushUsers() {
         List<PushUserInfo> l = new LinkedList<PushUserInfo>();
         for (Map.Entry<SimpleKey, DovecotPushListener> entry : listeners.entrySet()) {
             SimpleKey key = entry.getKey();
